/*
 *  Copyright (C) Chaperon. All rights reserved.
 *  -------------------------------------------------------------------------
 *  This software is published under the terms of the Apache Software License
 *  version 1.1, a copy of which has been included  with this distribution in
 *  the LICENSE file.
 */

package net.sourceforge.chaperon.grammar.token;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Vector;

import net.sourceforge.chaperon.grammar.Grammar;
import net.sourceforge.chaperon.grammar.symbol.Symbol;
import net.sourceforge.chaperon.grammar.symbol.SymbolList;

/**
 * This class contains a list of tokens. Like<br/>
 * identifier := [A-Za-z][A-Za-z0-9_]<br/>
 * number := [0-9]+
 *
 * @author Stephan Michels
 * @version $Revision: 1.7 $ $Date: 2002/05/07 11:01:49 $
 */
public class TokenList implements Serializable, Cloneable
{

	private Grammar _owner = null;
	private Vector _list = new Vector();

	/**
	 * Creates a empty list of tokens
	 */
	public TokenList()
	{
	}

	/**
	 * This method, design for th callback from the grammar
	 *
	 * @param owner The grammar, with owns the token list
	 */
	public void setOwner(Grammar owner)
	{
		if (_owner != null)
			throw new IllegalStateException("TokenList has already a owner");
		_owner = owner;
	}

	/**
	 * Return the owner of this list
	 *
	 * @return The grammar, with owns the token list
	 */
	public Grammar getOwner()
	{
		return _owner;
	}

	/**
	 * Add a token to this list.
	 * The method also executes the setOwner method from the token.
	 *
	 * @param token Token, which should be added
	 *
	 * @return Index of the token in this list
	 */
	public int addToken(Token token)
	{
    if (token==null)
      throw new NullPointerException();
	
		if (contains(token.getSymbol()))
  		throw new IllegalArgumentException("The token list owns already a token with the symbol '"+token.getSymbol()+"'");
	  token.setOwner(this);

		_list.addElement(token);
		return _list.size() - 1;
	}

	/**
	 * Add the tokens of an other list to this list
	 *
	 * @param list List of tokens
	 */
	public void addToken(TokenList list)
	{
		for (int i = 0; i < list.getTokenCount(); i++)
    {
      try
      {
  			addToken((Token)list.getToken(i).clone());
      } catch (CloneNotSupportedException cnse) 
      {
        throw new IllegalArgumentException("Could not clone token:"+cnse.getMessage());
      }
    }
	}

	/**
	 * Removes a token by an index from this list
	 *
	 * @param index Index of the token
	 */
	public void removeToken(int index)
	{
		_list.removeElementAt(index);
	}

	/**
	 * Removes the token which the symbol from this list
	 *
	 * @param tsymbol Terminal symbol of the token
	 */
	public void removeToken(Symbol tsymbol)
	{
		for (int i = _list.size() - 1; i >= 0; i--)
			if (((Token) _list.elementAt(i)).getSymbol().equals(tsymbol))
				_list.removeElementAt(i);
	}

	/**
	 * Set a token by an index
	 *
	 * @param index Index of the token
	 * @param token Token
	 */
	public void setToken(int index, Token token)
	{
		_list.setElementAt(token, index);
	}

	/**
	 * Return a token giving by an index
	 *
	 * @param index Index of token
	 *
	 * @return Token
	 */
	public Token getToken(int index)
	{
		if ((index >= 0) && (index < _list.size()))
			return (Token) _list.elementAt(index);
		return null;
	}

	/**
	 * Return a token giving by a symbol
	 *
	 * @param tsymbol Terminal symbol of the token
	 *
	 * @return Token
	 */
	public Token getToken(Symbol tsymbol)
	{
		int index = indexOf(tsymbol);

		if (index >= 0)
			return (Token) _list.elementAt(indexOf(tsymbol));
		return null;
	}

	/**
	 * Returns the count of tokens in the list
	 *
	 * @return Count of tokens
	 */
	public int getTokenCount()
	{
		return _list.size();
	}

	/**
	 * Return the index of a token
	 *
	 * @param token Token
	 *
	 * @return Index of the token
	 */
	public int indexOf(Token token)
	{
		for (int i = 0; i < _list.size(); i++)
			if (((Token) _list.elementAt(i)).equals(token))
				return i;
		return -1;
	}

	/**
	 * Return the index of a token, given by a terminal symbol
	 *
	 * @param tsymbol Terminal symbol of the token
	 *
	 * @return Index of the token
	 */
	public int indexOf(Symbol tsymbol)
	{
		for (int i = 0; i < _list.size(); i++)
			if (((Token) _list.elementAt(i)).getSymbol().equals(tsymbol))
				return i;
		return -1;
	}

	/**
	 * If the list contains a token
	 *
	 * @param token Token
	 *
	 * @return True, if this contains the token
	 */
	public boolean contains(Token token)
	{
		return (indexOf(token) != -1);
	}

	/**
	 * If the list contains a token with the given symbol
	 *
	 * @param tsymbol Terminal symbol of the token
	 *
	 * @return True, if this contains the token
	 */
	public boolean contains(Symbol tsymbol)
	{
		return (indexOf(tsymbol) != -1);
	}

	/**
	 * Removes all tokens from this list
	 */
	public void removeAllToken()
	{
		_list.removeAllElements();
	}

	/**
	 * Returns a enumeration of the tokens
	 * in this list.
	 *
	 * @return Enumeration of the tokens
	 */
	public Enumeration enumerateToken()
	{
		return _list.elements();
	}

	/**
	 * Returns the content of this list
	 *
	 * @return Array of the tokens
	 */
	public Token[] getToken()
	{
		int size = _list.size();
		Token[] mArray = new Token[size];

		for (int index = 0; index < size; index++)
			mArray[index] = (Token) _list.elementAt(index);
		return mArray;
	}

	/**
	 * Replace the content of this list by the content of an array
	 *
	 * @param tokenArray Array of tokens
	 */
	public void setToken(Token[] tokenArray)
	{
		_list.removeAllElements();
		for (int i = 0; i < tokenArray.length; i++)
			_list.addElement(tokenArray[i]);
	}

	/**
	 * Return all used symbols in this list
	 *
	 * @return List of Symbols
	 */
	public SymbolList getSymbols()
	{
		SymbolList list = new SymbolList(true);

		for (int i = 0; i < getTokenCount(); i++)
			list.addSymbol(getToken(i).getSymbol());
		return list;
	}

	/**
	 * Converts the list to a string
	 *
	 * @return Content of this list
	 */
	public String toString()
	{
		StringBuffer buffer = new StringBuffer();

		buffer.append("Tokens:\n");
		for (int i = 0; i < getTokenCount(); i++)
		{
			buffer.append(i + ".Token: ");
			buffer.append(getToken(i).toString());
			buffer.append("\n");
		}

		buffer.append("\n");
		return buffer.toString();
	}

	/**
	 *
	 *
	 * @return
	 *
	 * @throws CloneNotSupportedException
	 */
	public Object clone() throws CloneNotSupportedException
	{
		TokenList clone = new TokenList();

		for (int i = 0; i < _list.size(); i++)
			clone.addToken((Token) ((Token) _list.elementAt(i)).clone());

		return clone;
	}
}
