/*
 *  Copyright (C) Chaperon. All rights reserved.                               
 *  ------------------------------------------------------------------------- 
 *  This software is published under the terms of the Apache Software License 
 *  version 1.1, a copy of which has been included  with this distribution in 
 *  the LICENSE file.                                                         
 */

package net.sourceforge.chaperon.helpers;

/**
 * This class contains a list of integers
 *
 * @author Stephan Michels
 * @version CVS $Id: IntegerList.java,v 1.2 2002/03/12 13:20:50 benedikta Exp $
 */
public class IntegerList
{
  private boolean unit = false;

  private int capacityIncrement = 100;

  private int elementCount = 0;
  private int[] list = new int[10];

  private int dummy;

  /**
   * Constructs a empty list
   */
  public IntegerList()
  {
  }

  /**
   * If unit is true, this list contains no multiple
   * instances of the same value
   *
   * @param unit True, this list contains no multiple
   *             instances of the same value
   */
  public IntegerList(boolean unit)
  {
    this.unit = unit;
  }

  /**
   * Add a value to the list
   *
   * @param value Integer value
   *
   * @return Index of this value
   */
  public int add(int value)
  {
    if (unit)
    {
      int index = indexOf(value);

      if (index == -1)
      {
        ensureCapacity(elementCount + 1);
        list[elementCount++] = value;
        return elementCount - 1;
      }
      else
        return index;
    }

    ensureCapacity(elementCount + 1);
    list[elementCount++] = value;
    return elementCount - 1;
  }

  /**
   * Add the values of an other integer list
   *
   * @param list List of integer values
   */
  public void add(IntegerList list)
  {
    for (int i = 0; i < list.getSize(); i++)
      add(list.get(i));
  }

  /**
   * Add the values of an array
   *
   * @param array Array of integer values
   */
  public void add(int[] array)
  {
    for (int i = 0; i < array.length; i++)
      add(array[i]);
  }

  /**
   * Removes a value giving by an index
   *
   * @param index Index of the integer value
   */
  public void remove(int index)
  {
    if (index >= elementCount)
      throw new ArrayIndexOutOfBoundsException(index);

    elementCount--;
    if (index < elementCount)
      System.arraycopy(list, index + 1, list, index, elementCount - index);
    list[elementCount] = 0;
  }

  /**
   * Replace a value
   *
   * @param index Index of the value, which should be replaced
   * @param value The new value
   */
  public void set(int index, int value)
  {
    if (index >= elementCount)
      throw new ArrayIndexOutOfBoundsException(index);

    list[index] = value;
  }

  /**
   * Return a value from this list
   *
   * @param index Index of the value
   *
   * @return Integer value
   */
  public int get(int index)
  {
    if (index >= elementCount)
      throw new ArrayIndexOutOfBoundsException(index);

    return list[index];
  }

  /**
   * Returns the count of value in this list
   *
   * @return Count of integer values
   */
  public int getSize()
  {
    return elementCount;
  }

  /**
   * Return the index of a value, otherwise -1
   *
   * @param value Value, which should found in this list
   *
   * @return Index of this value
   */
  public int indexOf(int value)
  {
    for (int i = 0; i < elementCount; i++)
    {
      if (list[i] == value)
        return i;
    }
    return -1;
  }

  /**
   * If the list contains a value
   *
   * @param value Value, which should found in this list
   *
   * @return True, if this list contains the value
   */
  public boolean contains(int value)
  {
    return (indexOf(value) != -1);
  }

  /**
   * Removes a value from this list
   *
   * @param value Value to remove
   */
  public void removeValue(int value)
  {
    int index;

    while ((index = indexOf(value)) != -1)
      remove(index);
  }

  /**
   * If this list contains no values
   *
   * @return True, if this list is empty
   */
  public boolean empty()
  {
    return (elementCount <= 0);
  }

  /**
   * Pops a value of the end of this list
   *
   * @return Value
   */
  public int pop()
  {
    if (0 >= elementCount)
      throw new java.util.EmptyStackException();

    // System.out.println("list[elementCount="+(elementCount-1)+"]="+list[elementCount-1]);
    return list[--elementCount];
  }

  /**
   * Pushs a value to this list
   *
   * @param value Value
   */
  public void push(int value)
  {
    add(value);
  }

  /**
   * Push values from an array
   *
   * @param values Array of integer values
   */
  public void push(int[] values)
  {
    add(values);
  }

  /**
   * Peek a value of the end of this list
   *
   * @return Integer value
   */
  public int peek()
  {
    if (0 >= elementCount)
      throw new java.util.EmptyStackException();

    return list[elementCount - 1];
  }

  /**
   * Removes all values from this list
   */
  public void clear()
  {
    elementCount = 0;
  }

  /**
   * Swaps two values from this list
   *
   * @param index1 Index from the first value
   * @param index2 Index from the second value
   */
  public void swap(int index1, int index2)
  {
    dummy = list[index1];
    list[index1] = list[index2];
    list[index2] = dummy;
  }

  /**
   * Convert the list to a string
   *
   * @return Content of the list
   */
  public String toString()
  {
    StringBuffer buffer = new StringBuffer();

    buffer.append("[");
    for (int i = 0; i < elementCount; i++)
    {
      buffer.append(String.valueOf(list[i]));
      if (i < (elementCount - 1))
        buffer.append(",");
    }
    buffer.append("]");
    return buffer.toString();
  }

  /**
   * Ensure the capacity for adding values
   *
   * @param minCapacity
   */
  private void ensureCapacity(int minCapacity)
  {
    if (list.length >= minCapacity)
      return;

    int newCapacity = list.length + capacityIncrement;

    if (capacityIncrement <= 0)
      newCapacity = list.length * 2;

    int[] newArray = new int[Math.max(newCapacity, minCapacity)];

    System.arraycopy(list, 0, newArray, 0, list.length);
    list = newArray;
  }
}
